@@ -1,9 +1,10 @@
 import gzip
 import re
 import numpy as np
-import pandas as pd
 from pathlib import Path
 from metaerg import context
+from metaerg.datatypes import sqlite
+from metaerg.datatypes.functional_genes import format_list_of_subsystem_genes
 
 NON_IUPAC_RE_NT = re.compile(r'[^ACTGN]')
 NON_IUPAC_RE_AA = re.compile(r'[^RHKDESTNQCUGPAVILMFYW]')
@@ -79,25 +80,18 @@ class FastaParser:
 
 
 class Masker:
-    def __init__(self, feature_data, targets=None, min_length=50):
-        if targets:
-            self.feature_data = feature_data[feature_data['type'].isin(targets)]
-            self.apply_mask = True
-        else:
-            self.feature_data = feature_data
-            self.apply_mask = False
-        self.min_length = min_length
+    def __init__(self, db_connection, targets=None):
+        self.db_connection = db_connection
+        self.targets = targets
         self.nt_total = 0
         self.nt_masked = 0
 
     def mask(self, seq_record):
         seq = seq_record['seq']
-        if self.apply_mask:
-            feature_data = self.feature_data.loc[lambda df: df['contig'] == seq_record['id'], :]
-            for feature in feature_data.itertuples():
-                feature_length = feature.end - feature.start
-                seq = seq[:feature.start] + 'N' * feature_length + seq[feature.end:]
-                self.nt_masked += feature_length
+        if self.targets:
+            for feature in sqlite.read_all_features(self.db_connection,  contig=seq_record['id'], type=self.targets):
+                seq = seq[:feature.start] + 'N' * feature.length_nt() + seq[feature.end:]
+                self.nt_masked += feature.length_nt()
 
         self.nt_total += len(seq)
 
@@ -120,7 +114,7 @@ def write_fasta(handle, fasta, line_length=80):
         pass
     try:
         if fasta["subsystems"]:
-            handle.write(f' ({fasta["subsystems"]})')
+            handle.write(f' {format_list_of_subsystem_genes(fasta["subsystems"])}')
     except KeyError:
         pass
     try:
@@ -134,10 +128,8 @@ def write_fasta(handle, fasta, line_length=80):
         handle.write('\n')
 
 
-def write_features_to_fasta(feature_data: pd.DataFrame, seq_type: str, base_file: Path, split=1, targets = None):
-    if targets:
-        feature_data = feature_data[feature_data['type'].isin(targets)]
-    number_of_records = len(feature_data.index)
+def write_features_to_fasta(db_connection, seq_type: str, base_file: Path, split=1, targets = None):
+    number_of_records = sum(1 for f in sqlite.read_all_features(db_connection, type=targets))
     split = min(split, number_of_records)
     records_per_file = number_of_records / split if split else number_of_records
     if split > 1:
@@ -146,25 +138,25 @@ def write_features_to_fasta(feature_data: pd.DataFrame, seq_type: str, base_file
         paths = base_file,
     filehandles = [open(p, 'w') for p in paths]
     records_written = 0
-    for feature in feature_data.itertuples():
-        fasta_rec = {'id': feature.id, 'descr': feature.descr}
+    for feature in sqlite.read_all_features(db_connection, type=targets):
+        fasta_rec = {'id': feature.id, 'descr': feature.descr, 'subsystems': feature.subsystems, 'taxon': feature.taxon}
         if 'aa' == seq_type:
             fasta_rec['seq'] = feature.aa_seq
         elif 'nt' == seq_type:
             fasta_rec['seq'] = feature.nt_seq
         write_fasta(filehandles[int(records_written / records_per_file)], fasta_rec)
         records_written += 1
-    if not len(feature_data):
+    if not number_of_records:
         context.log(f'WARNING: No [{", ".join(targets)}] features in genome, '
                     f'fasta file(s) {", ".join([str(p.name) for p in paths])} are empty!')
     return paths
 
 
-def write_contigs_to_fasta(contig_dict: dict, base_file: Path, feature_data: pd.DataFrame = None, genome_name='',
-                           split=1, mask_targets=None, mask_min_length=50):
+def write_contigs_to_fasta(contig_dict: dict, base_file: Path, db_connection, genome_name='',
+                           split=1, mask_targets=None):
     """writes contigs to fasta file(s), optionally masking features with N"""
     number_of_records = len(contig_dict)
-    split = min(split, number_of_records)
+    split = int(min(split, number_of_records))
     records_per_file = number_of_records / split
     if split > 1:
         paths = [Path(base_file.parent, f'{base_file.name}.{i}') for i in range(split)]
@@ -172,7 +164,7 @@ def write_contigs_to_fasta(contig_dict: dict, base_file: Path, feature_data: pd.
         paths = base_file,
     filehandles = [open(p, 'w') for p in paths]
     records_written = 0
-    masker = Masker(feature_data, targets=mask_targets, min_length=mask_min_length)
+    masker = Masker(db_connection, targets=mask_targets)
     for contig in contig_dict.values():
         write_fasta(filehandles[int(records_written / records_per_file)], masker.mask(contig))
         records_written += 1
@@ -181,3 +173,43 @@ def write_contigs_to_fasta(contig_dict: dict, base_file: Path, feature_data: pd.
     if mask_targets:
         context.log(f'({genome_name}) {masker.stats()}')
     return paths
+
+
+
+def load_contigs(genome_name, input_fasta_file, delimiter=context.DELIMITER, rename_contigs=context.RENAME_CONTIGS,
+                 min_contig_length=context.MIN_CONTIG_LENGTH):
+    if delimiter in genome_name:
+        raise Exception(f'Genome id {genome_name} contains "{delimiter}"; change delimiter with '
+                                f'--delimiter [new delimiter] or use --rename_genomes')
+    names_done = set()
+    contigs = list()
+    with FastaParser(input_fasta_file) as fasta_reader:
+        for c in fasta_reader:
+            if len(c['seq']) < min_contig_length:
+                continue
+            c['descr'] = ''  # remove descr as it makes parsing results more difficult for some analyses
+            contigs.append(c)
+            if not rename_contigs:
+                if c['id'] in names_done:
+                    raise Exception(f'Contig id {c["id"]} not unique. Use --rename_contigs to avoid this problem.')
+                names_done.add(c["id"])
+    contigs.sort(key=lambda c:len(c['seq']), reverse=True)
+    total_length = sum(len(c['seq']) for c in contigs)
+    context.log(f'({genome_name}) Loaded {len(contigs)} contigs with total length {total_length:,} from file.')
+    if rename_contigs:
+        contig_name_mapping_file = context.spawn_file('contig.name.mappings', genome_name)
+        context.log(f'({genome_name}) Renaming contigs (see {contig_name_mapping_file})...')
+        i = 0
+        with open(contig_name_mapping_file, 'w') as mapping_writer:
+            for c in contigs:
+                new_id = f'{genome_name}.c{i:0>4}'
+                mapping_writer.write(f'{c["id"]}\t{new_id}\n')
+                c['id'] = new_id
+                i += 1
+    else:
+        for c_id in contigs:
+            if delimiter in c_id:
+                raise Exception(f'Contig id {c_id} contains "{delimiter}"; change delimiter with '
+                                f'--delimiter [new delimiter] or use --rename_contigs')
+    contigs = {c['id']: c for c in contigs}
+    return contigs
