@@ -1,16 +1,18 @@
 from pathlib import Path
-import pandas as pd
+
 from metaerg import context
-from metaerg.datatypes.blast import taxon_at_genus, DBentry, BlastHit, BlastResult
+from metaerg.datatypes.functional_genes import format_list_of_subsystem_genes
+from metaerg.datatypes import sqlite
+from metaerg.datatypes.blast import taxon_at_genus
 
 
 @context.register_html_writer
-def write_html(genome_name, feature_data: pd.DataFrame, genome_properties:dict, dir):
+def write_html(genome_name, db_connection, genome_properties:dict, dir):
     dir.mkdir(exist_ok=True, parents=True)
     file = Path(dir, genome_name, "feature_table.html")
     file.parent.mkdir(exist_ok=True, parents=True)
     with open(Path(file), 'w') as handle:
-        handle.write(make_html(genome_name, feature_data, genome_properties))
+        handle.write(make_html(genome_name, db_connection, genome_properties))
 
 
 def get_empty_format_dict():
@@ -29,7 +31,7 @@ def get_empty_format_dict():
             'ci': '', 'ca': '', 'cr': '', 'ct': ''}
 
 
-def format_feature(f, format_hash, dominant_taxon, colors, path_to_feature_html):
+def format_feature(f: sqlite.Feature, format_hash, top_taxon, colors):
     format_hash['f_id'] = f.id
     format_hash['taxon'] = taxon_at_genus(f.taxon)
     format_hash['type'] = f.type
@@ -56,22 +58,18 @@ def format_feature(f, format_hash, dominant_taxon, colors, path_to_feature_html)
             format_hash['destination'] = 'cytoplasm'
         case [*_]:
             format_hash['destination'] = ''
-    format_hash['subsystem'] = f.subsystems
+    format_hash['subsystem'] = format_list_of_subsystem_genes(f.subsystems)
     format_hash['has_cdd'] = 'Y' if f.cdd is not None else ''
-    try:
-        if len(f.blast):
-            blast_result = eval(f.blast)
-            format_hash['ident'] = f'{blast_result.hits[0].percent_id:.1f}'
-            format_hash['ci'] = colors[min(int(blast_result.hits[0].percent_id / 20), len(colors) - 1)]
-            format_hash['align'] = f'{blast_result.percent_aligned():.1f}'
-            format_hash['ca'] = colors[min(int(blast_result.percent_aligned() / 20), len(colors) - 1)]
-            format_hash['recall'] = f'{blast_result.percent_recall():.1f}'
-            format_hash['cr'] = colors[min(int(blast_result.percent_recall() / 20), len(colors) - 1)]
-    except SyntaxError:
-        print(f.blast)
-    dominant_taxon = dominant_taxon.split()
+    if f.blast:
+        format_hash['ident'] = f'{f.blast.hits[0].percent_id:.1f}'
+        format_hash['ci'] = colors[min(int(f.blast.hits[0].percent_id / 20), len(colors) - 1)]
+        format_hash['align'] = f'{f.blast.percent_aligned():.1f}'
+        format_hash['ca'] = colors[min(int(f.blast.percent_aligned() / 20), len(colors) - 1)]
+        format_hash['recall'] = f'{f.blast.percent_recall():.1f}'
+        format_hash['cr'] = colors[min(int(f.blast.percent_recall() / 20), len(colors) - 1)]
+    top_taxon = top_taxon.split()
     taxon = f.taxon.split()
-    format_hash['ct'] = colors[int(len(colors) * len(set(taxon) & set(dominant_taxon)) / (len(taxon) + 1))]
+    format_hash['ct'] = colors[int(len(colors) * len(set(taxon) & set(top_taxon)) / (len(taxon) + 1))]
 
 
 def format_hash_to_html(format_hash):
@@ -83,7 +81,7 @@ def format_hash_to_html(format_hash):
     </tr>'''.format(**format_hash)
 
 
-def make_html(genome_name, feature_data: pd.DataFrame, genome_properties:dict, path_to_feature_html='') -> str:
+def make_html(genome_name, db_connection, genome_properties:dict) -> str:
     """Injects the content into the html base, returns the html."""
     html = _make_html_template()
     html = html.replace('GENOME_NAME', genome_name)
@@ -101,8 +99,8 @@ def make_html(genome_name, feature_data: pd.DataFrame, genome_properties:dict, p
     table_body = ''
     previous_repeats = []
     prev_f = None
-    for f in feature_data.itertuples():
-        if f.type in ('crispr_repeat', 'repeat') and (not len(previous_repeats) or (f and f.type is prev_f.type)):
+    for f in sqlite.read_all_features(db_connection):
+        if f.type in ('crispr_repeat', 'repeat') and (not len(previous_repeats) or (f and f.type == prev_f.type)):
             previous_repeats.append(f)
         elif len(previous_repeats):
             format_hash = get_empty_format_dict()
@@ -116,11 +114,11 @@ def make_html(genome_name, feature_data: pd.DataFrame, genome_properties:dict, p
                 previous_repeats.append(f)
             else:
                 format_hash = get_empty_format_dict()
-                format_feature(f, format_hash, genome_properties['dominant taxon'], colors, path_to_feature_html)
+                format_feature(f, format_hash, genome_properties['classification (top taxon)'], colors)
                 table_body += format_hash_to_html(format_hash)
         else:
             format_hash = get_empty_format_dict()
-            format_feature(f, format_hash, genome_properties['dominant taxon'], colors, path_to_feature_html)
+            format_feature(f, format_hash, genome_properties['classification (top taxon)'], colors)
             table_body += format_hash_to_html(format_hash)
         prev_f = f
     html = html.replace('TABLE_BODY', table_body)
