@@ -1,59 +1,41 @@
 from pathlib import Path
-import pandas as pd
-from metaerg import context
 
-GENOME_PROPERTY_FORMATS = {'size': ',',
-                           '% GC': '.1%',
-                           'N50': ',',
-                           '# proteins': ',',
-                           '% coding': '.1%',
-                           'mean protein length (aa)': '.1f',
-                           '# ribosomal RNA': ',',
-                           '# transfer RNA': ',',
-                           '# non-coding RNA': ',',
-                           '# retrotransposons': ',',
-                           '# CRISPR repeats': ',',
-                           '# other repeats': ',',
-                           '# total features': ',',
-                           '% repeats': '.1%',
-                           '% CDS classified to taxon': '.1%',
-                           '% of CDS classified to dominant taxon': '.1%',
-                           'dominant taxon': '<',
-                           'codon usage bias': '.3f',
-                           'doubling_time (days)': '.1f'
-                           }
+from metaerg import context
+from metaerg.datatypes.genome_properties import GENOME_PROPERTY_FORMATS
 
 @context.register_html_writer
-def write_html(genome_name, feature_data: pd.DataFrame, genome_properties:dict, dir):
+def write_html(genome_name, db_connection, genome_properties:dict, dir):
     dir.mkdir(exist_ok=True, parents=True)
     file = Path(dir, genome_name, 'index.html')
     with open(file, 'w') as handle:
-        handle.write(make_html(genome_name, feature_data, genome_properties))
+        handle.write(make_html(genome_name, genome_properties))
 
 
-def make_html(genome_name, feature_data: pd.DataFrame, genome_properties:dict, path_to_feature_html='') -> str:
+def make_html(genome_name, genome_properties:dict) -> str:
     """injects the content into the html base, returns the html"""
     html = _make_html_template()
     html = html.replace('GENOME_NAME', genome_name)
     # genome properties
     html = html.replace('CONTENT_PROPERTIES', ''.join((f'<tr><td>{k}</td><td>{genome_properties[k]:{GENOME_PROPERTY_FORMATS[k]}}</td></tr>\n'
                                                        for k in GENOME_PROPERTY_FORMATS.keys())))
+    # subsystem_summary
+    sssummary = genome_properties['subsystem_summary']
+    html = html.replace('CONTENT_SUBSYSTEM_SUMMARY', ''.join((f'<tr><td>{k}</td><td><a href="#{k}">{v:{"," if isinstance(v, int) else ".1%"}}</a></td></tr>\n'
+                                                             for k, v in sssummary.items() if (isinstance(v, float) and v>=0.5) or isinstance(v, int) and v>0)))
+    # subsystem_data
     subsystem_html = ''
     subsystem_data = genome_properties['subsystems']
-    # subsystem_data
-    for subsystem in subsystem_data.index.unique('subsystem'):
-        sub_data = subsystem_data.loc[subsystem, :]
-        subsystem_html += f'<b id=f>{subsystem}</b>'
-        if 'secondary-metabolites' == subsystem:
-            if len(sub_data.index):
+    for subsystem, subsystem_genes in subsystem_data.items():
+        subsystem_html += f'<a id="{subsystem}"><b id=f>{subsystem}</b></a>'
+        if 'Secondary-metabolites' == subsystem:
+            if len(subsystem_genes):
                 subsystem_html += '<p id=f><a href="antismash/index.html" target="">View antismash results.</a></p>\n'
         else:
             subsystem_html += '<table id=f>\n'
-            for function in sub_data.itertuples():
-                if function.Index:
-                    subsystem_html += f'<tr><td>{function.Index}</td><td>\n'
-                    subsystem_html += ', '.join('<a target="Gene Details" href="feature-details.html#{}">{}</a>'
-                                                .format(f_id.split("@")[0], f_id) for f_id in function.genes.split())
+            for gene_name, feature_ids in subsystem_genes.items():
+                subsystem_html += f'<tr><td>{gene_name}</td><td>\n'
+                subsystem_html += ', '.join('<a target="Gene Details" href="feature-details.html#{}">{}</a>'
+                                            .format(f_id, f_id) for f_id in feature_ids)
                 subsystem_html += f'</td></tr>\n'
             subsystem_html += '</table>\n'
     html = html.replace('CONTENT_SUBSYSTEMS', subsystem_html)
@@ -104,6 +86,8 @@ def _make_html_template() -> str:
   }
 </style>
 
+<h4 id=f><a href="feature_table.html">(View table with all genes here)</a></h4>
+<br>
 <h3 id=f>Genome properties</h4>
 <table id=f>
     <thead>
@@ -114,10 +98,10 @@ def _make_html_template() -> str:
     </thead>
     <tbody>
 CONTENT_PROPERTIES
+CONTENT_SUBSYSTEM_SUMMARY
     </tbody></table>
     <p></p>
-    <h4 id=f><a href="feature_table.html">View table with all genes</a></h4>
     <p></p>
-    <h3 id=f>Subsystems overview</h3>
+    <h3 id=f>Subsystems in Detail</h3>
 CONTENT_SUBSYSTEMS
 </body></html>'''
