@@ -1,23 +1,25 @@
 import shutil
 import os
-import pandas as pd
 from pathlib import Path
 
 from metaerg import context
 from metaerg.datatypes import gbk
+from metaerg.datatypes import sqlite
+from metaerg.datatypes import functional_genes
 
 
-def _run_programs(genome_name, contig_dict, feature_data: pd.DataFrame, result_files):
+def _run_programs(genome_name, contig_dict, db_connection, result_files):
     """Should execute the helper programs to complete the analysis"""
     gbk_file = context.spawn_file('gbk', genome_name)
     if context.MULTI_MODE:
         gbk_file = Path(gbk_file.parent, gbk_file.name + '.gbk')
     with open(gbk_file, 'w') as handle:
-        gbk.gbk_write_genome(handle, contig_dict, feature_data)
+        gbk.gbk_write_genome(handle, contig_dict, db_connection)
+    shutil.rmtree(result_files[0])
     context.run_external(f'antismash --genefinding-tool none --output-dir {result_files[0]} {gbk_file}')
 
 
-def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_files) -> tuple:
+def _read_results(genome_name, contig_dict, db_connection, result_files) -> int:
     """Should parse the result files and return the # of positives."""
     antismash_hit_count = 0
     for f in sorted(result_files[0].glob('*region*.gbk')):
@@ -25,7 +27,6 @@ def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_f
             antismash_region_name = ''
             antismash_region_number = 0
             for f_as_dict in reader:
-                antismash_hit_count += 1
                 if 'region' == f_as_dict['type']:
                     antismash_region_name = '{} {}'.format(f_as_dict['product'], f_as_dict['rules'])
                     antismash_region_number = int(f_as_dict['region_number'])
@@ -33,30 +34,23 @@ def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_f
                     antismash_gene_function = f_as_dict.get('gene_functions', '')
                     antismash_gene_category = f_as_dict.get('gene_kind', '')
                     if antismash_region_name:
-                        try:
-                            if not f_as_dict['locus_tag'] in feature_data.index:
-                                # antismash sometimes predicts a new ORF, for example for a lassopeptide
-                                # we do not add this to the feature table
-                                continue
-                        except KeyError:
+                        feature = sqlite.read_feature_by_id(db_connection, f_as_dict['locus_tag'])
+                        if not feature:
+                            # antismash sometimes predicts a new ORF, for example for a lassopeptide
+                            # we do not add this to the feature table
                             continue
-                        feature_data.at[f_as_dict['locus_tag'], 'antismash'] = \
-                            ' '.join((f'(region {antismash_region_number})', antismash_region_name,
-                                      antismash_gene_function, antismash_gene_category))
-                        prev_subsystem_text = feature_data.at[f_as_dict['locus_tag'], 'subsystems']
-                        if type(prev_subsystem_text) == str and '[secondary-metabolites]' not in prev_subsystem_text:
-                                feature_data.at[f_as_dict['locus_tag'], 'subsystems'] += ' [secondary-metabolites]'
-                        elif f_as_dict['locus_tag'] in feature_data.index:
-                            print(f"starting: '{f_as_dict['locus_tag']}'")
-                            feature_data.at[f_as_dict['locus_tag'], 'subsystems'] = '[secondary-metabolites]'
-    if not antismash_hit_count:
-        result_files[0].mkdir(exist_ok=True)  # to prevent re-doing fruitless searches
-    return feature_data, antismash_hit_count
+                        feature.antismash =  ' '.join((f'(region {antismash_region_number})', antismash_region_name,
+                                                       antismash_gene_function, antismash_gene_category))
+                        feature.subsystems.append(functional_genes.SECONDARY_METABOLITE_GENE)
+                        sqlite.update_feature_in_db(db_connection, feature)
+                        antismash_hit_count += 1
+    return antismash_hit_count
 
 
 @context.register_annotator
 def run_and_read_antismash():
     return ({'pipeline_position': 91,
+             'annotator_key': 'antismash',
              'purpose': 'prediction of secondary metabolite genes with antismash',
              'programs': ('antismash',),
              'result_files': ('antismash',),
@@ -65,7 +59,7 @@ def run_and_read_antismash():
 
 
 @context.register_html_writer
-def write_html(genome_name, feature_data: pd.DataFrame, genome_properties:dict, dir):
+def write_html(genome_name, db_connection, genome_properties:dict, dir):
     """need to copy the antismash result dir to the metaerg html dir."""
     dir.mkdir(exist_ok=True, parents=True)
     antismash_result_dir = context.spawn_file('antismash', genome_name)
@@ -78,7 +72,7 @@ def write_html(genome_name, feature_data: pd.DataFrame, genome_properties:dict,
 
 @context.register_database_installer
 def format_antismash_databases():
-    if 'A' not in context.TASKS:
+    if 'A' not in context.DATABASE_TASKS:
         return
     antismash_database_path = context.DATABASE_DIR / 'antismash'
     context.log(f'Installing antismash database at {antismash_database_path}')
