@@ -1,7 +1,6 @@
 import re
 import gzip
 import shutil
-import pandas as pd
 from os import chdir
 from ftplib import FTP
 from pathlib import Path
@@ -13,6 +12,7 @@ import ncbi.datasets
 
 from metaerg import context
 from metaerg.datatypes import fasta
+from metaerg.datatypes import sqlite
 from metaerg.datatypes.blast import BlastResult, DBentry, TabularBlastParser
 from metaerg.datatypes import ncbi_ftp
 
@@ -30,14 +30,13 @@ IGNORED_FEATURE_TYPES = 'gene pseudogene exon direct_repeat region sequence_feat
                         'transcript 3\'UTR 5\'UTR intron signal_peptide_region_of_CDS sequence_alteration'.split()
 
 
-def _run_programs(genome_name, contig_dict, feature_data: pd.DataFrame, result_files):
+def _run_programs(genome_name, contig_dict, db_connection, result_files):
     rna_nt_file = context.spawn_file('rna.fna', genome_name)
     blastn_result_file = context.spawn_file('blastn', genome_name)
     if rna_nt_file.exists() and rna_nt_file.stat().st_size:
-        if context.FORCE or not blastn_result_file.exists() or not blastn_result_file.stat().st_size:
-            blastn_db = Path(context.DATABASE_DIR, DB_RNA_FILENAME)
-            context.run_external(f'blastn -db {blastn_db} -query {rna_nt_file} -out {blastn_result_file} -max_target_seqs 10 '
-                                 f'-outfmt 6')
+        blastn_db = Path(context.DATABASE_DIR, DB_RNA_FILENAME)
+        context.run_external(f'blastn -db {blastn_db} -query {rna_nt_file} -out {blastn_result_file} -max_target_seqs 10 '
+                             f'-outfmt 6')
     else:
         context.log(f'({genome_name}) Skipping blastn, fasta file with RNA genes missing or empty.')
 
@@ -47,7 +46,7 @@ def _run_programs(genome_name, contig_dict, feature_data: pd.DataFrame, result_f
                          f'--threads {context.CPUS_PER_GENOME} --fast --max-target-seqs 10')
 
 
-def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_files) -> tuple:
+def _read_results(genome_name, contig_dict, db_connection, result_files) -> int:
     # (1) load databse descriptions
     db_descr = Path(context.DATABASE_DIR, DB_DESCRIPTIONS_FILENAME)
     descriptions = {'p': {}, 'e': {}, 'v': {}}
@@ -73,12 +72,14 @@ def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_f
     # (3) parse diamond blast results
 
     def process_blast_result(blast_result: BlastResult):
-        if blast_result.query() not in feature_data.index:
+        feature = sqlite.read_feature_by_id(db_connection, blast_result.query())
+        if not feature:
             raise Exception(f'Found results for unknown feature {blast_result.query()}, '
                             f'may need to rerun metaerg with --force')
-        feature_data.at[blast_result.query(), 'blast'] = str(blast_result)
-        feature_data.at[blast_result.query(), 'descr'] = blast_result.hits[0].hit.descr
-        feature_data.at[blast_result.query(), 'taxon'] =  blast_result.hits[0].hit.taxon
+        feature.blast = blast_result
+        feature.descr = blast_result.hits[0].hit.descr
+        feature.taxon =  blast_result.hits[0].hit.taxon
+        sqlite.update_feature_in_db(db_connection, feature)
 
     def dbentry_from_string(db_id: str) -> DBentry:
         w = db_id.split('~')
@@ -96,12 +97,13 @@ def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_f
         for blast_result in handle:
             blast_result_count += 1
             process_blast_result(blast_result)
-    return feature_data, blast_result_count
+    return blast_result_count
 
 
 @context.register_annotator
 def run_and_read_diamond_blastn():
     return ({'pipeline_position': 81,
+             'annotator_key': 'diamond_and_blastn',
              'purpose': 'function prediction and taxonomic classification of genes with diamond and blastn',
              'programs': ('diamond', 'blastn'),
              'databases': (Path(DB_PROTEINS_FILENAME + '.dmnd'),
@@ -137,12 +139,12 @@ def update_db_descriptions_get_db_id(description, dictionary, file_handle, kingd
 
 @context.register_database_installer
 def install_viral_database():
-    if 'V' not in context.TASKS:
+    if 'V' not in context.DATABASE_TASKS:
         return
     context.log('Downloading viral refseq from the NCBI...')
     VIR_DB_DIR = Path(context.DATABASE_DIR, 'ncbi-cache', 'vir')
     VIR_DB_DIR.mkdir(exist_ok=True, parents=True)
-    if Path(VIR_DB_DIR, DB_DESCRIPTIONS_FILENAME).exists() and not context.FORCE:
+    if Path(VIR_DB_DIR, DB_DESCRIPTIONS_FILENAME).exists() and not context.DATABASE_FORCE:
         context.log(f'Keeping existing viral database in {VIR_DB_DIR}, use --force to overwrite.')
         return
     fasta_protein_db, fasta_nt_db, descr_db, taxon_db = init_pristine_db_dir(VIR_DB_DIR)
@@ -190,12 +192,12 @@ def install_viral_database():
 
 @context.register_database_installer
 def install_eukaryote_database():
-    if 'E' not in context.TASKS:
+    if 'E' not in context.DATABASE_TASKS:
         return
     context.log('Downloading taxon-unique eukaryotic genomes using NCBI Datasets...')
     EUK_DB_DIR = Path(context.DATABASE_DIR, 'ncbi-cache', 'euk')
     EUK_DB_DIR.mkdir(exist_ok=True, parents=True)
-    if Path(EUK_DB_DIR, DB_DESCRIPTIONS_FILENAME).exists() and not context.FORCE:
+    if Path(EUK_DB_DIR, DB_DESCRIPTIONS_FILENAME).exists() and not context.DATABASE_FORCE:
         context.log(f'Keeping existing eukaryote database in {EUK_DB_DIR}, use --force to overwrite.')
         return
     fasta_protein_db, fasta_nt_db, descr_db, taxon_db = init_pristine_db_dir(EUK_DB_DIR)
@@ -337,7 +339,7 @@ def install_eukaryote_database():
 
 @context.register_database_installer
 def install_prokaryote_database():
-    if 'P' not in context.TASKS:
+    if 'P' not in context.DATABASE_TASKS:
         return
     context.log('Downloading a gff file from the NCBI FTP server for each prokaryotic genome in gtdbtk...')
     PROK_DB_DIR = Path(context.DATABASE_DIR, 'ncbi-cache', 'pro')
@@ -346,7 +348,7 @@ def install_prokaryote_database():
     PROK_DB_DIR_FAA.mkdir(exist_ok=True)
     PROK_DB_DIR_FNA = Path(PROK_DB_DIR, 'fna')
     PROK_DB_DIR_FNA.mkdir(exist_ok=True)
-    if Path(PROK_DB_DIR, DB_DESCRIPTIONS_FILENAME).exists() and not context.FORCE:
+    if Path(PROK_DB_DIR, DB_DESCRIPTIONS_FILENAME).exists() and not context.DATABASE_FORCE:
         context.log(f'Keeping existing prokaryote database in {PROK_DB_DIR}, use --force to overwrite.')
         return
     RNA_DESCR_RE = re.compile(r'\[product=(.+?)]')
@@ -444,7 +446,7 @@ def compile_databases():
 
 @context.register_database_installer
 def format_blast_databases():
-    if 'B' not in context.TASKS:
+    if 'B' not in context.DATABASE_TASKS:
         return
     context.log('Concatenating and formatting blast databases for (prok, euk, vir)...')
     PROK_DB_DIR = Path(context.DATABASE_DIR, 'ncbi-cache', 'pro')
