@@ -1,24 +1,24 @@
 import os
 import gzip
 import shutil
-import pandas as pd
 from math import log10
 from pathlib import Path
 from concurrent import futures
 
 from metaerg.datatypes.blast import DBentry, TabularBlastParser
+from metaerg.datatypes import sqlite
 from metaerg import context
-from metaerg import functional_gene_configuration
+from metaerg.datatypes import functional_genes
 
 
-def _run_programs(genome_name, contig_dict, feature_data: pd.DataFrame, result_files):
+def _run_programs(genome_name, contig_dict, db_connection, result_files):
     cds_aa_file = context.spawn_file('cds.faa', genome_name)
     hmm_db = context.DATABASE_DIR / 'hmm' / 'functional_genes.hmm'
     # the domtbl format includes alignment starts and ends, so we can use that information...
     context.run_external(f'hmmscan -o /dev/null -E 1e-6 --domtblout {result_files[0]} {hmm_db} {cds_aa_file}')
 
 
-def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_files) -> tuple:
+def _read_results(genome_name, contig_dict, db_connection, result_files) -> int:
     db_entry = None
     hmm_db = context.DATABASE_DIR / 'hmm' / 'functional_genes.hmm'
     functional_gene_db = {}
@@ -46,11 +46,12 @@ def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_f
     with TabularBlastParser(result_files[0], 'HMMSCAN_DOM_TABLE', get_db_entry) as handle:
         hit_count = 0
         for blast_result in handle:
-            if blast_result.query() not in feature_data.index:
+            feature = sqlite.read_feature_by_id(db_connection, blast_result.query())
+            if not feature:
                 raise Exception(f'Found results for unknown feature {blast_result.query()}, '
                                 f'may need to rerun metaerg with --force')
             blast_result.hits = blast_result.hits[:10]
-            feature_data.at[blast_result.query(), 'hmm'] = str(blast_result)
+            feature.hmm = blast_result
             for h in blast_result.hits:
                 db_entry = h.hit
                 confidence = 1.0
@@ -63,17 +64,20 @@ def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_f
                 else:
                     if h.evalue > 0:
                         confidence = min(1.0, - log10(h.evalue) / 100)
-                if new_subsystem := functional_gene_configuration.match_hit(h, confidence):
+                if new_matches := functional_genes.match_hit(h, confidence):
                     hit_count += 1
-                    feature_data.at[blast_result.query(), 'subsystems'] = functional_gene_configuration.cleanup_subsystem_str(
-                        feature_data.at[blast_result.query(), 'subsystems'].strip() + ' ' + new_subsystem)
+                    for new_match in new_matches:
+                        if not new_match in feature.subsystems:
+                            feature.subsystems.append(new_match)
                 break
-    return feature_data, hit_count
+            sqlite.update_feature_in_db(db_connection, feature)
+    return hit_count
 
 
 @context.register_annotator
 def run_and_read_canthyd():
     return ({'pipeline_position': 101,
+             'annotator_key': 'hmm',
              'purpose': 'identification of functional genes with HMMs',
              'programs': ('hmmscan',),
              'databases': (Path('hmm', 'functional_genes.hmm'),),
@@ -125,19 +129,19 @@ def add_hmm_file_to_db(hmm_file: Path, db_handle, names_done: set, targets: set)
 
 @context.register_database_installer
 def install_functional_gene_databases():
-    if 'S' not in context.TASKS:
+    if 'S' not in context.DATABASE_TASKS:
         return
 
-    functional_gene_configuration.init_functional_gene_config()
+    functional_genes.init_functional_gene_config()
     all_target_hmm_names = set()
-    for t in functional_gene_configuration.SUBSYSTEM_DATA['profiles']:
-        for t1 in t.split('|'):
-            all_target_hmm_names.add(t1)
+    for gene_def in functional_genes.GENES:
+        all_target_hmm_names.update(gene_def.cues)
 
     hmm_dir = context.DATABASE_DIR / 'hmm'
     hmm_dir.mkdir(exist_ok=True, parents=True)
-    (hmm_dir / 'user_config').mkdir(exist_ok=True, parents=True)  # used by functional_gene_configuration.py
-    (hmm_dir / 'user_hmm').mkdir(exist_ok=True, parents=True)  # used below
+    user_hmm_dir = hmm_dir / 'user_hmm'
+    (hmm_dir / 'user_config').mkdir(exist_ok=True, parents=True)  # used by functional_genes.py
+    user_hmm_dir.mkdir(exist_ok=True, parents=True)  # used below
     context.log(f'Installing functional gene hmm database at {hmm_dir}...')
 
     current_working_dir = os.getcwd()
@@ -184,7 +188,7 @@ def install_functional_gene_databases():
         outcomes = []
         for url in FUNCTIONAL_GENE_URLS:
             destination_file = hmm_dir / Path(url).name
-            if context.FORCE or not destination_file.exists() or not destination_file.stat().st_size:
+            if context.DATABASE_FORCE or not destination_file.exists() or not destination_file.stat().st_size:
                 outcomes.append(executor.submit(context.download, url, destination_file))
         for future in futures.as_completed(outcomes):
             future.result()
@@ -198,7 +202,8 @@ def install_functional_gene_databases():
             if f == all_hmms_file:
                 continue
             add_hmm_file_to_db(hmm_file, output, names_done, all_target_hmm_names)
-        for f in (hmm_dir / 'user_hmm').glob('*.hmm'):
+        for f in user_hmm_dir.glob('*.hmm'):
             hmm_file = f.absolute()
             add_hmm_file_to_db(hmm_file, output, names_done, all_target_hmm_names)
     context.run_external(f'hmmpress -f {hmm_dir / "functional_genes.hmm"}')
+    os.chdir(current_working_dir)
